"""
Package where events related to the ``content_authoring`` subdomain are implemented.
"""
# Comment broken up to deal with line-length issues with the long URL

# The ``content_authoring`` subdomain corresponds to {Architecture Subdomain} defined in OEP-41.
# https://open-edx-proposals.readthedocs.io/en/latest/architectural-decisions/oep-0041-arch-async-server-event-messaging.html
